import StringIO
import collections


class MockHTTPConnection(object):
    """
    Mock class for HTTPConnection
    """
    requests = []

    def __init__(self, ip_address=None):
        with open('responses') as f:
            self.responses = collections.defaultdict(list)
            self.__load_responses(f)
        for key, value in self.responses.items():
            self.responses[key] = ''.join(value)

    def __load_responses(self, f):
        actual = ''
        for line in f:
            if line.startswith('#'):
                actual = line[1:].strip()
            else:
                self.responses[actual].append(line)

    def request(self, method, route, content):
        """
        Mock HTTP request. Adds the method and the route
        of the request to the list of mock requests
        """
        MockHTTPConnection.requests.append('%s %s' % (method, route))

    def getresponse(self):
        """
        Mock response. Returns the corresponding entry of the
        last mock request, or the error message if it does not exist
        """
        try:
            response = self.responses[MockHTTPConnection.requests[-1]]
        except KeyError:
            response = '{"error": {"description": <description>}}'
        finally:
            return StringIO.StringIO(response)



import re
import unittest
import pyhue


from mockhttpconnection import MockHTTPConnection
pyhue.HTTPConnection = MockHTTPConnection


class TestPyhue(unittest.TestCase):
    def setUp(self):
        MockHTTPConnection.requests = []
        self.bridge = pyhue.Bridge('0.0.0.0', 'username')

    def tearDown(self):
        self.bridge = None

    def assertMatches(self, string, pattern):
        return re.match(pattern, string) is not None


class TestLight(TestPyhue):
    def test_get_light(self):
        _ = self.bridge.get_light(1)
        self.assertMatches(MockHTTPConnection.requests[0],
                           'GET /api/(.+?)/lights/1')
    
    def test_get_lights(self):
        _ = self.bridge.lights
        self.assertMatches(MockHTTPConnection.requests[0],
                           'GET /api/(.+?)/lights')
        self.assertMatches(MockHTTPConnection.requests[1],
                           'GET /api/(.+?)/lights/1')

    def test_set_light_state(self):
        lights = self.bridge.lights
        lights[0].on = True
        self.assertMatches(MockHTTPConnection.requests[2],
                           'PUT /api/(.+?)/lights/1/state')

    def test_get_light_state(self):
        lights = self.bridge.lights
        _ = lights[0].on
        self.assertMatches(MockHTTPConnection.requests[1],
                           'GET /api/(.+?)/lights/1/state')

    def test_check_light_state(self):
        lights = self.bridge.lights
        lights[0].hue = 0
        self.assertEqual(lights[0].hue, 0)
        
    def test_update_light_state(self):
        lights = self.bridge.lights
        lights[0].hue = 0
        self.assertMatches(MockHTTPConnection.requests[2],
                           'PUT /api/(.+?)/lights/1/state')

    def test_set_light_attrs(self):
        lights = self.bridge.lights
        lights[0].name = "Another Mock Lamp"
        self.assertMatches(MockHTTPConnection.requests[2],
                           'PUT /api/(.+?)/lights/1')


class TestGroups(TestPyhue):
    def test_get_groups(self):
        _ = self.bridge.groups
        self.assertMatches(MockHTTPConnection.requests[0],
                           'GET /api/(.+?)/groups')
        self.assertMatches(MockHTTPConnection.requests[1],
                           'GET /api/(.+?)/groups/1')

    def test_get_num_groups(self):
        groups = self.bridge.groups
        self.assertEqual(len(groups), 2)

    def test_set_group_state(self):
        groups = self.bridge.groups
        groups[0].on = True
        self.assertMatches(MockHTTPConnection.requests[2],
                           'PUT /api/(.+?)/groups/1/action')

    def test_check_group_state(self):
        groups = self.bridge.groups
        groups[0].hue = 0
        self.assertEqual(groups[0].hue, 0)

    def test_set_group_attrs(self):
        groups = self.bridge.groups
        groups[0].name = "Another Mock Group"
        self.assertMatches(MockHTTPConnection.requests[2],
                           'PUT /api/(.+?)/groups/1')


class TestSchedules(TestPyhue):
    def test_get_schedules(self):
        _ = self.bridge.schedules
        self.assertMatches(MockHTTPConnection.requests[0],
                           'GET /api/(.+?)/schedules')
        self.assertMatches(MockHTTPConnection.requests[1],
                           'GET /api/(.+?)/schedules/1')

    def test_add_schedule(self):
        schedule_attrs = {
            "name": "Another Schedule Name",
            "description": "",
            "command": {
                "address": "/api/username/groups/1/action",
                "method": "PUT",
                "body": {"on": True}
            },
            "time": "2006-10-14T22:30:00"
        }
        self.bridge.add_schedule(schedule_attrs)
        self.assertMatches(MockHTTPConnection.requests[0],
                           'POST /api/(.+?)/schedules')

    def test_set_schedule_attrs(self):
        schedules = self.bridge.schedules
        schedules[0].name = "Another Schedule Name"
        self.assertMatches(MockHTTPConnection.requests[2],
                           'PUT /api/(.+?)/schedules/1')

    def test_delete_schedule(self):
        schedules = self.bridge.schedules
        del schedules[0]
        self.assertMatches(MockHTTPConnection.requests[2],
                           'DELETE /api/(.+?)/schedules/1')



#!/usr/bin/env python
# -*- coding: utf-8 -*-

import json
try:
    from httplib import HTTPConnection
except ImportError:
    from http.client import HTTPConnection


kelvin2mired = lambda k: 10**6 / k
huedegree2hue = lambda d: d* 182


def cmyk2hsb(c, m, y, k):
    pass


def rgb2xy(r, g, b):
    X = 0.412453 * r + 0.357580 * g + 0.180423 * b
    Y = 0.212671 * r + 0.715160 * g + 0.072169 * b
    Z = 0.019334 * r + 0.119193 * g + 0.950227 * b
    x = X / (X+Y+Z)
    y = Y / (X+Y+Z)
    return x, y


class HueException(Exception):
    pass


class Bridge(object):
    def __init__(self, ip_address, username):
        self.ip_address = ip_address
        self.username = username

    def _request(self, method, route, data={}):
        content = json.dumps(data).lower()
        str_route = '/'.join(['/api', self.username] + route)
        try:
            conn = HTTPConnection(self.ip_address)
            conn.request(method, str_route, content)
            response = conn.getresponse()
            return json.loads(response.read().decode())
        except:
            raise HueException("Not able to connect to the bridge")

    def __get_api_objects(self, cls):
        result = self._request('GET', [cls.ROUTE])
        objects = [cls(self, i) for i in result.keys()]
        return sorted(objects, key=lambda x: x.id)

    @property
    def lights(self):
        return self.__get_api_objects(Light)

    @property
    def groups(self):
        return [Group(self, "0")] + self.__get_api_objects(Group)

    @property
    def schedules(self):
        return self.__get_api_objects(Schedule)
    
    def get_light(self, light_id):
        return Light(self, light_id)

    def add_schedule(self, schedule_attrs):
        return self._request('POST', ['schedules'], schedule_attrs)


class AssignableSetattr(type):
    def __new__(mcls, name, bases, attrs):  # @NoSelf
        def __setattr__(self, attr, value):
            object.__setattr__(self, attr, value)

        init_attrs = dict(attrs)
        init_attrs['__setattr__'] = __setattr__
        base = super(AssignableSetattr, mcls)
        init_cls = base.__new__(mcls, name, bases, init_attrs)
        real_cls = base.__new__(mcls, name, (init_cls,), attrs)
        init_cls.__real_cls = real_cls
        return init_cls

    def __call__(cls, *args, **kwargs):  # @NoSelf
        self = super(AssignableSetattr, cls).__call__(*args, **kwargs)
        real_cls = cls.__real_cls
        self.__class__ = real_cls
        return self


class ApiObject(object):
    __metaclass__ = AssignableSetattr

    def __init__(self, bridge, _id):
        result = bridge._request('GET', [self.ROUTE, _id])
        if any('error' in x for x in result):
            raise HueException(result[0]['error']['description'])
        self.bridge = bridge
        self.id = _id
        for attr, value in result.items():
            setattr(self, attr, value)

    def set(self, attr, value):
        object.__setattr__(self, attr, value)
        api_route = [self.ROUTE, self.id]
        return self.bridge._request('PUT', api_route, {attr: value})


class Light(ApiObject):
    ROUTE = 'lights'

    def update(self, **kw):
        self.state.update(kw)
        api_route = [self.ROUTE, self.id, 'state']
        return self.bridge._request('PUT', api_route, kw)

    def __setattr__(self, attr, value):
        if attr in self.state:
            result = self.update(**{attr: value})
        else:
            result = self.set(attr, value)
        if any('error' in confirmation for confirmation in result):
            raise HueException("Invalid attribute")

    def __getattr__(self, attr):
        if attr in self.state:
            return self.state[attr]
        else:
            return object.__getattr__(self, attr)


class Group(ApiObject):
    ROUTE = 'groups'

    def update(self, **kw):
        self.action.update(kw)
        api_route = [self.ROUTE, self.id, 'action']
        return self.bridge._request('PUT', api_route, kw)

    def __setattr__(self, attr, value):
        if attr in self.action:
            result = self.update(**{attr: value})
        else:
            result = self.set(attr, value)
        if any('error' in confirmation for confirmation in result):
            raise HueException("Invalid attribute")

    def __getattr__(self, attr):
        if attr in self.action:
            return self.action[attr]
        else:
            return object.__getattribute__(self, attr)


class Schedule(ApiObject):
    ROUTE = 'schedules'

    def __setattr__(self, attr, value):
        result = self.set(attr, value)
        if any('error' in confirmation for confirmation in result):
            raise HueException("Invalid attribute")

    def __del__(self):
        self.bridge._request('DELETE', [self.ROUTE, self.id])



#!/usr/bin/env python

from distutils.core import setup


long_description = '''
pyhue
=====

Python library for the Philips Hue personal lighting system.

Installation
------------

You can install pyhue with ``pip install pyhue`` or `download pyhue.py <http://raw.github.com/alexrdp90/pyhue/master/src/pyhue.py>`_ and place it in your project directory.


Example
-------

::

    import pyhue
    
    bridge = pyhue.Bridge('my_ip_address', 'my_username')
    for light in bridge.lights:
        light.on = True
        light.hue = 0


Features
--------

- Object-oriented mapping of the RESTful interface.
- Major support of the v1.0 of the API: Lights_, Groups_, Schedules_.
- Conversion of basic color models.


See the complete documentation of the Philips Hue personal lighting system on http://developers.meethue.com.

.. _Lights: http://developers.meethue.com/1_lightsapi.html
.. _Groups: http://developers.meethue.com/2_groupsapi.html
.. _Schedules: http://developers.meethue.com/3_schedulesapi.html

'''


setup(
    name = 'pyhue',
    version = '0.7.5',
    license = 'GPLv3',
    description = 'Python library for the Philips Hue personal lighting system',
    long_description = long_description,
    author = 'Alejandro Rodas',
    author_email = 'alexrdp90@gmail.com',
    url = 'http://github.com/alexrdp90/pyhue',
    download_url = 'http://pypi.python.org/pypi/pyhue',
    package_dir = {'': 'src'},
    py_modules = ['pyhue'],
    classifiers = [
        'Development Status :: 3 - Alpha',
        'License :: OSI Approved :: GNU General Public License v3 (GPLv3)',
        'Operating System :: OS Independent',
        'Programming Language :: Python',
        'Programming Language :: Python :: 2.7',
        'Topic :: Software Development :: Libraries :: Python Modules',
    ],
)